local HeadCount = LibStub("AceAddon-3.0"):GetAddon("HeadCount2")
if not HeadCount then
	print("Failed to load Boss Detection, we cant find HeadCount")
	return
end

local plugin = HeadCount:NewModule("BossDetection","AceEvent-3.0","AceTimer-3.0")
local BB = LibStub("LibBabble-Boss-3.0"):GetUnstrictLookupTable()

local CallbackHandler = LibStub:GetLibrary("CallbackHandler-1.0")

plugin.Callbacks = CallbackHandler:New(plugin)

local UnitExists = UnitExists
local UnitAffectingCombat = UnitAffectingCombat
local UnitName = UnitName
local UnitClassification = UnitClassification
local InstanceBossList = {}
local KilledBossList = {}
local InstanceIndex = 0
local monitoring = false
local engagedBossName = nil
local engagedBosses = {}
local targetTable = {"target", "targettarget", "focus", "focustarget", "mouseover", "mouseovertarget"}
local BossIsEngaged = false
local rriHandle = nil
local scanHandle = nil

local function DebugEvent(event,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10)
	print(event,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10)
end

function plugin:OnInitialize()
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA","CheckInstance")
	self:RegisterEvent("PLAYER_ENTERING_WORLD","CheckInstance")
	self:RegisterEvent("RAID_ROSTER_UPDATE","CheckInstance")
	self:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT", DebugEvent)
	self:RegisterEvent("UNIT_TARGETABLE_CHANGED",DebugEvent)
	self:RegisterEvent("UPDATE_INSTANCE_INFO","CheckFlexLocks")
	self:BuildTargetTable()
end

function plugin:OnEnabled()
	-- Lets check our data right now
end

function plugin:CheckInstance(event)
	print(event)
	-- whenever we change zones, or enter an instance restart monitoring
	if event == "ZONE_CHANGED_NEW_AREA" or "PLAYER_ENTERING_WORLD" then
		self:StopMonitoring()
	end
	-- Otherwise once in a raid/party
	if UnitInRaid("player") or GetNumPartyMembers() > 1 then
		RequestRaidInfo()
		self:StartMonitoring()
	else
		self:StopMonitoring()
	end
end

function plugin:GetInstanceBossList()
	if IsInInstance() then
		local lockTimeleft, isPreviousInstance, encountersTotal, encountersComplete = GetInstanceLockTimeRemaining();
		for i=1,encountersTotal do
			bossName, texture, isKilled = GetInstanceLockTimeRemainingEncounter(i);
			InstanceBossList[bossName] = i
			KilledBossList[bossName] = false
		end
	end
end

function plugin:GetSavedInstanceData()
	if InstanceIndex == 0 then
		for k,v in pairs(GetFullRaidList()) do
			local 	name, id, reset, difficulty, locked, extended, instanceIDMostSig, isRaid, maxPlayers, difficultyName, numEncounters, encounterProgress = GetSavedInstanceInfo(k)
			local  bossName,text,isKilled = GetSavedInstanceEncounterInfo(k,1)
			if InstanceBossList[bossName] then
				InstanceIndex = k
				-- Get the status of the active raid kill list
				for m=1,numEncounters do
					bossName,text,isKilled = GetSavedInstanceEncounterInfo(k,m)
					KilledBossList[bossName] = isKilled
				end
			end
		end
	else
		local 	name, id, reset, difficulty, locked, extended, instanceIDMostSig, isRaid, maxPlayers, difficultyName, numEncounters, encounterProgress = GetSavedInstanceInfo(InstanceIndex)
		for m=1,numEncounters do
			bossName,text,isKilled = GetSavedInstanceEncounterInfo(InstanceIndex,m)
			KilledBossList[bossName] = isKilled
		end
	end
end

function plugin:StartMonitoring()
	if monitoring then return end
	-- Get the list of instance bosses
	if IsInInstance() then
		self:GetInstanceBossList()
		self:GetSavedInstanceData()
	end
	monitoring = true
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "ExitedCombat")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "EnteredCombat")
end

function plugin:StopMonitoring()
	monitoring = false
	table.wipe(InstanceBossList)
	table.wipe(KilledBossList)
	table.wipe(engagedBosses)
	engagedBossName = nil
	InstanceIndex = 0
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	self:UnregisterEvent("PLAYER_REGEN_DISABLED")
end

local function IsBossUnitEngaged(unit)
	if UnitExists(unit) then
		local uName = UnitName(unit)
		if UnitAffectingCombat(unit) and UnitClassification(unit) == "worldboss" then
			if BB[uName] and HeadCount.BOSS_BLACKLIST[BB[uName]] then
				return false
			elseif BB[uName] and HeadCount.BOSS_WHITELIST[BB[uName]] then
				return true
			else
				return true
			end
		end
	end
	return false
end

function plugin:BuildTargetTable()
	for i = 1, 4 do targetTable[#targetTable+1] = string.format("boss%d", i) end
	for i = 1, 4 do targetTable[#targetTable+1] = string.format("party%dtarget", i) end
	for i = 1, 40 do targetTable[#targetTable+1] = string.format("raid%dtarget", i) end
end

local fBosses = {}
local function FindBossUnits()
	table.wipe(fBosses)
	for i, unit in next, targetTable do
		if UnitExists(unit) and not UnitIsPlayer(unit) and UnitClassification(unit) == "worldboss"then
			local name = UnitName(unit)
			local lName = BB[name]
			if not fBosses[name] then
				if lName and not HeadCount.BOSS_BLACKLIST[lName] then
					tinsert(engagedBosses,lName)
					fBosses[lName] = true
				elseif not lName then -- We dont have a localized version of the boss name
					HeadCount:LogWarning("Boss "..name.." missing locaization")
					tinsert(engagedBosses,name)
					fBosses[name] = true
				end
			end
		end
	end
end
local function GetBossUnit(bossName)
	for i, unit in next, targetTable do
		if UnitExists(unit) and not UnitIsPlayer(unit) then
			local uName = UnitName(unit)
			if uName == bossName then
				return unit
			end
		end
	end
end

function plugin:Scan()
	-- Find all WorldBossUnits
	FindBossUnits()
	local engaged = false
	for k,v in pairs(engagedBosses) do
		local unit = GetBossUnit(v)
		if unit and UnitAffectingCombat(unit) then
			engaged = true
		end
	end
	return engaged
end

function plugin:ScanForEngage()
	if self:Scan() and not BossEngaged then
		BossIsEngaged = true
		self:DetermineEncounter()
	else
		scanHandle = self:ScheduleTimer(plugin.ScanForEngage,0.5,self)
	end
end

function plugin:ScanForWipe()
	if not self:Scan() then
		BossIsEngaged = false
		-- No units in combat anymore, lets see if the raid lock updated.
		if IsInInstance() then
			-- Lets check raid locks
			RequestRaidInfo()
			print("Asked for Flex Locks")
		else
			-- World boss, lets see if he is dead.
			local unit = GetBossUnit(engagedBossName)
			if UnitIsDead(unit) then
				-- Success
				print(engagedBossName," has been defeated!")
				self.Callbacks:Fire("BossKilled",engagedBossName)
				engagedBossName = nil
			else
				-- Wipe!
				print(engagedBossName," has wiped your raid!")
				self.Callbacks:Fire("BossWipe",engagedBossName)
				engagedBossName = nil
			end
		end
	else
		self:ScheduleTimer(plugin.ScanForWipe,0.5,self)
	end
end


function plugin:DetermineEncounter()
	if engagedBossName then return end
	local foundEncounter = false
	local bossCount = 0
	-- Only 1 boss unit present
	if #engagedBosses == 1 then
		engagedBossName = engagedBosses[1]
		if BB[engagedBossName] then
			engagedBossName = BB[engagedBossName]
		end
	end
	-- Multiple Boss Units present, find the alias
	for k,v in pairs(engagedBosses) do
		local localizedBoss = BB[v]
		if localizedBoss then
			if HeadCount.BOSS_ALIASES[localizedBoss] then
				engagedBossName = localizedBoss
			end
		end
	end
	-- We didnt have an alias for this encounter and it isnt a single boss
	if not engagedBossName then
		HeadCount:LogWarning(string.format("No Alias found for this multiboss encounter! Please report this to HeadCount2 project %s",table.concat(engagedBosses, ",")))
	else
		print(engagedBossName," Encounter has been started.")
		self.Callbacks:Fire("BossEngaged",engagedBossName)
	end
end

function plugin:ExitedCombat()
	self:CancelTimer(scanHandle,true)
	self:ScanForWipe()
end

function plugin:EnteredCombat()
	table.wipe(engagedBosses)
	self:ScanForEngage()
end

local tmpBossList = {}

function plugin:CheckFlexLocks()
	if not monitoring then return end
	if not engagedBossName then return 	end
	if BossIsEngaged then return end
	if rriHandle then rriHandle = nil end

	table.wipe(tmpBossList)
	for k,v in pairs(KilledBossList) do
		tmpBossList[k] = v
	end
	self:GetSavedInstanceData()
	for k,v in pairs(KilledBossList) do
		if v ~= tmpBossList[k] then
			-- We have an updated lock
			print(engagedBossName," has been defeated!")
			self.Callbacks:Fire("BossKilled",engagedBossName)
			engagedBossName = nil
		end
	end
	-- Boss wasnt in the updated lock, lets see if we wiped
	if engagedBossName and not BossIsEngaged then
		local unit = GetBossUnit(engagedBossName)
		if #engagedBosses > 1 then
			for k,v in pairs(engagedBosses) do
				unit = GetBossUnit(v)
				if unit and not UnitIsDead(unit) then
					print(engagedBossName," has wiped your raid!")
					self.Callbacks:Fire("BossWipe",engagedBossName)
					engagedBossName = nil
				end
			end
		else
			if unit and not UnitIsDead(unit) then
				-- Wipe!
				print(engagedBossName," has wiped your raid!")
				self.Callbacks:Fire("BossWipe",engagedBossName)
				engagedBossName = nil
			end
		end
	end
end